@@ -19,7 +19,8 @@
 #' 
 #' @param M Genotype in numeric format, pure 0, 1, 2 matrix; m * n, m is marker size, n is population size
 #' @param K kinship matrix
-#' @param priority speed or memory
+#' @param maxLine the number of markers handled at a time, smaller value would reduce the memory cost
+#' @param ind_idx the index of effective genotyped individuals used in analysis
 #' @param pcs.keep maximum number of PCs for output
 #' @param cpu the number of cpu
 #' @param verbose whether to print detail.
@@ -40,23 +41,13 @@
 #' }
 #' 
 MVP.PCA <-
-function(M=NULL, K=NULL, priority=c("speed", "memory"), pcs.keep=5, cpu=1, verbose=TRUE){
-    # R.ver <- Sys.info()[['sysname']]
-    # wind <- R.ver == 'Windows'
-    # linux <- R.ver == 'Linux'
-    # mac <- (!linux) & (!wind)
-    # r.open <- eval(parse(text = "!inherits(try(Revo.version,silent=TRUE),'try-error')"))
+function(M=NULL, K=NULL, maxLine=10000, ind_idx=NULL, pcs.keep=5, cpu=1, verbose=TRUE){
 
-    # if (r.open && mac) {
-    #     Sys.setenv("VECLIB_MAXIMUM_THREADS" = "1")
-    # }
     #Data Check
     if (is.null(M) & is.null(K)) {
         stop("There is no genotype data or relationship matrix!")
     }
 
-    priority <- match.arg(priority)
-
     if (pcs.keep < 0) {
         pcs.keep = 0
     }
@@ -66,11 +57,12 @@ function(M=NULL, K=NULL, priority=c("speed", "memory"), pcs.keep=5, cpu=1, verbo
     }
 
     if(is.null(K)){
-        K <- MVP.K.VanRaden(M=M, priority=priority, cpu=cpu)
+        K <- MVP.K.VanRaden(M=M, ind_idx = ind_idx, maxLine = maxLine, cpu = cpu, verbose = verbose)
+    }else{
+        K <- K[ind_idx, ind_idx]
     }
 
     logging.log("Eigen Decomposition on GRM", "\n", verbose = verbose)
-    # if(r.open)  eval(parse(text = "try(setMKLthreads(cpu), silent=TRUE)"))
     PCs <- eigen(K, symmetric=TRUE)$vectors[, 1:pcs.keep]
     logging.log("Deriving PCs successfully", "\n", verbose = verbose)
 
